import bpy
from .gltf2_extension.texture_video import texture_video
from .gltf2_extension.materials_animation import materials_animation
from .gltf2_extension.material_properties import material_properties
from .gltf2_extension.materials_variants import materials_variants
from .gltf2_extension.custom_properties import custom_properties
from .gltf2_extension.lightmap import lightmap
from .gltf2_extension.shader_mix import shader_mix

gltf2_extensions = [
    texture_video,
    materials_animation,
    material_properties,
    materials_variants,
    custom_properties,
    lightmap,
    shader_mix
]

bl_info = {
    'name': '山海鲸',
    'description': '山海鲸插件合集。',
    'version': (0, 2, 0),
    'blender': (3, 3, 0),
    'category': '山海鲸',
    'author': '山海鲸可视化',
    'wiki_url': 'https://www.shanhaibi.com/',
}

class ShanhaibiProperties(bpy.types.PropertyGroup):
    enabled: bpy.props.BoolProperty(
        name = '山海鲸',
        description = '导出山海鲸扩展',
        default = True
    )

def register():
    bpy.utils.register_class(ShanhaibiProperties)
    bpy.types.Scene.ShanhaibiProperties = bpy.props.PointerProperty(type = ShanhaibiProperties)
    for gltf2_extension in gltf2_extensions:
        if hasattr(gltf2_extension, 'register'):
            gltf2_extension.register()

def register_panel():
    try:
        bpy.utils.register_class(GLTF_PT_ShanhaibiPanel)
    except Exception:
        pass
    return unregister_panel

def unregister_panel():
    try:
        bpy.utils.unregister_class(GLTF_PT_ShanhaibiPanel)
    except Exception:
        pass

def unregister():
    for gltf2_extension in gltf2_extensions:
        if hasattr(gltf2_extension, 'unregister'):
            gltf2_extension.unregister()
    bpy.utils.unregister_class(ShanhaibiProperties)
    del bpy.types.Scene.ShanhaibiProperties

class GLTF_PT_ShanhaibiPanel(bpy.types.Panel):
    bl_space_type = 'FILE_BROWSER'
    bl_region_type = 'TOOL_PROPS'
    bl_label = "山海鲸"
    bl_parent_id = "GLTF_PT_export_user_extensions"

    @classmethod
    def poll(cls, context):
        sfile = context.space_data
        operator = sfile.active_operator
        return operator.bl_idname == "EXPORT_SCENE_OT_gltf"

    def draw_header(self, context):
        self.layout.prop(bpy.context.scene.ShanhaibiProperties, 'enabled', text='')

    def draw(self, context):
        layout = self.layout
        layout.active = bpy.context.scene.ShanhaibiProperties.enabled
        layout.use_property_split = True
        layout.use_property_decorate = False
        for gltf2_extension in gltf2_extensions:
            if hasattr(gltf2_extension, 'pannel_draw'):
                gltf2_extension.pannel_draw(self, context)



class glTF2ExportUserExtension:
    def __init__(self):
        from io_scene_gltf2.io.com.gltf2_io_extensions import Extension
        self.Extension = Extension
        self.shanhaibi_props = bpy.context.scene.ShanhaibiProperties
        for gltf2_extension in gltf2_extensions:
            if hasattr(gltf2_extension, 'user_extension_init'):
                gltf2_extension.user_extension_init(self)

    def gather_texture_hook(self, gltf2_texture, blender_shader_sockets, export_settings):
        if not self.shanhaibi_props.enabled: return
        for gltf2_extension in gltf2_extensions:
            if hasattr(gltf2_extension, 'gather_texture_hook'):
                gltf2_extension.gather_texture_hook(self, gltf2_texture, blender_shader_sockets, export_settings)

    def gather_material_hook(self, gltf2_material, blender_material, export_settings):
        if not self.shanhaibi_props.enabled: return
        for gltf2_extension in gltf2_extensions:
            if hasattr(gltf2_extension, 'gather_material_hook'):
                gltf2_extension.gather_material_hook(self, gltf2_material, blender_material, export_settings)

    def gather_mesh_hook(self, gltf2_mesh, blender_mesh, blender_object, vertex_groups, modifiers, skip_filter, material_names, export_settings):
        if not self.shanhaibi_props.enabled: return
        for gltf2_extension in gltf2_extensions:
            if hasattr(gltf2_extension, 'gather_mesh_hook'):
                gltf2_extension.gather_mesh_hook(self, gltf2_mesh, blender_mesh, blender_object, vertex_groups, modifiers, skip_filter, material_names, export_settings)

    def gather_node_hook(self, gltf2_node, blender_object, export_settings):
        if not self.shanhaibi_props.enabled: return
        for gltf2_extension in gltf2_extensions:
            if hasattr(gltf2_extension, 'gather_node_hook'):
                gltf2_extension.gather_node_hook(self, gltf2_node, blender_object, export_settings)

    def gather_gltf_extensions_hook(self, gltf2_plan, export_settings):
        if not self.shanhaibi_props.enabled: return
        for gltf2_extension in gltf2_extensions:
            if hasattr(gltf2_extension, 'gather_gltf_extensions_hook'):
                gltf2_extension.gather_gltf_extensions_hook(self, gltf2_plan, export_settings)