# 可视化的方式设置mesh的自定义属性

import bpy

# Properties
class MeshProperties(bpy.types.PropertyGroup):
    castshadow: bpy.props.BoolProperty(
        name = '产生阴影',
        description = '开启时物体会产生阴影',
        default = True
    )
    receiveshadows: bpy.props.BoolProperty(
        name = '接收阴影',
        description = '开启时物体会接收阴影',
        default = True
    )
    reflectenvironment: bpy.props.BoolProperty(
        name = '反射环境',
        description = '开启时物体会反射环境',
        default = False
    )



# Panel
class MeshPanel(bpy.types.Panel):
    bl_label = '扩展属性'
    bl_idname = 'MESH_PT_EXTRA_PROPERTIES'
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = 'data'

    def draw(self, context):
        props = context.mesh.DSProperties_Properties
        self.layout.prop(props, 'castshadow')
        self.layout.prop(props, 'receiveshadows')
        self.layout.prop(props, 'reflectenvironment')



# Register / Unregister
def register():
    bpy.utils.register_class(MeshProperties)
    bpy.utils.register_class(MeshPanel)
    bpy.types.Mesh.DSProperties_Properties = bpy.props.PointerProperty(type = MeshProperties)

def unregister():
    bpy.utils.unregister_class(MeshProperties)
    bpy.utils.unregister_class(MeshPanel)
    del bpy.types.Mesh.DSProperties_Properties



def gather_mesh_hook(user_extension, gltf2_mesh, blender_mesh, blender_object, vertex_groups, modifiers, skip_filter, material_names, export_settings):
    props = blender_mesh.DSProperties_Properties

    if not gltf2_mesh.extras:
        gltf2_mesh.extras = {}
    gltf2_mesh.extras['castShadow'] = props.castshadow
    gltf2_mesh.extras['receiveShadows'] = props.receiveshadows
    gltf2_mesh.extras['reflectEnvironment'] = props.reflectenvironment