# 支持光照贴图

import bpy

extension_is_required = False
glTF_extension_name = "DS_lightmap"

import nodeitems_utils
from nodeitems_utils import NodeCategory, NodeItem
from bpy.types import Node

class DSCategory(NodeCategory):
    @classmethod
    def poll(cls, context):
        return context.space_data.tree_type == 'ShaderNodeTree'

node_categories = [
    DSCategory("DS_NODES", "山海鲸", items=[
        NodeItem("lightmap.node")
    ]),
]

class LightmapNode(Node):
    bl_idname = 'lightmap.node'
    bl_label = '光照贴图'
    bl_icon = 'LIGHT'
    bl_width_min = 216.3
    bl_width_max = 330.0

    intensity: bpy.props.FloatProperty(name="Intensity", soft_min=0, soft_max=1, default=1)

    def init(self, context):
        lightmap = self.inputs.new('NodeSocketColor', "Lightmap")
        lightmap.hide_value = True

        self.width = 216.3

    @classmethod
    def poll(cls, ntree):
        return ntree.bl_idname == 'ShaderNodeTree'

    def draw_buttons(self, context, layout):
        layout.prop(self, "intensity")

    def draw_label(self):
        return "光照贴图"

def register():
    bpy.utils.register_class(LightmapNode)
    nodeitems_utils.register_node_categories("DS_NODES", node_categories)

def unregister():
    bpy.utils.unregister_class(LightmapNode)
    nodeitems_utils.unregister_node_categories("DS_NODES")


def gather_lightmap_texture_info(blender_material, export_settings):
    from io_scene_gltf2.blender.exp import gltf2_blender_gather_texture, gltf2_blender_gather_texture_info
    from io_scene_gltf2.io.com.gltf2_io import TextureInfo
    nodes = blender_material.node_tree.nodes
    lightmap_node = next(
        (n for n in nodes if isinstance(n, LightmapNode)), None)

    if not lightmap_node:
        return

    texture_socket = lightmap_node.inputs.get("Lightmap")
    intensity = lightmap_node.intensity

    texture, factor = gltf2_blender_gather_texture.gather_texture((texture_socket,), export_settings)
    if bpy.app.version < (3, 2, 0):
        tex_transform, tex_coord = gltf2_blender_gather_texture_info.__gather_texture_transform_and_tex_coord(
            texture_socket, export_settings)
    else:
        tex_transform, tex_coord, _ = gltf2_blender_gather_texture_info.__gather_texture_transform_and_tex_coord(
            texture_socket, export_settings)
    texture_info = TextureInfo(
        extensions=gltf2_blender_gather_texture_info.__gather_extensions(
            tex_transform, export_settings),
        extras=None,
        index=texture,
        tex_coord=tex_coord
    )

    if not texture_info:
        return

    return {
        "intensity": intensity,
        'extensions': texture_info.extensions,
        'extras': texture_info.extras,
        "index": texture_info.index,
        "texCoord": texture_info.tex_coord
    }


def gather_material_hook(user_extension, gltf2_object, blender_material, export_settings):
    if blender_material.node_tree and blender_material.use_nodes:
        lightmap_texture_info = gather_lightmap_texture_info(blender_material, export_settings)
        if lightmap_texture_info:
            gltf2_object.extensions[glTF_extension_name] = user_extension.Extension(
                name = glTF_extension_name,
                extension = lightmap_texture_info,
                required = extension_is_required,
            )