# 导出视频材质

import bpy

extension_is_required = False
glTF_extension_name = "DS_texture_video"

class VideoTextureProperties(bpy.types.PropertyGroup):
    enabled: bpy.props.BoolProperty(
        name = '视频材质',
        description = '导出视频材质',
        default = True
    )

def register():
    bpy.utils.register_class(VideoTextureProperties)
    bpy.types.Scene.VideoTextureProperties = bpy.props.PointerProperty(type = VideoTextureProperties)


def unregister():
    bpy.utils.unregister_class(VideoTextureProperties)
    del bpy.types.Scene.VideoTextureProperties


def pannel_draw(panel, context):
    layout = panel.layout
    layout.prop(bpy.context.scene.VideoTextureProperties, 'enabled')



from os import path
from io_scene_gltf2.blender.exp import gltf2_blender_export_keys
from io_scene_gltf2.io.exp import gltf2_io_binary_data
from io_scene_gltf2.io.exp import gltf2_io_image_data

def tmp_encode_movie(image):
    path = image.filepath_from_user()
    with open(path, "rb") as f:
        encoded_image = f.read()
    return encoded_image

def gather_mimetype(image):
    _name, extension = path.splitext(image.filepath)
    if extension in [".avi", ".mov", ".mp4", ".webm"]:
        return {
            ".avi": "video/avi",
            ".mov": "video/quicktime",
            ".mp4": "video/mp4",
            ".webm": "video/webm"
        }[extension]
    return None

def gather_buffer_view(image_data: bytes, export_settings):
    if export_settings[gltf2_blender_export_keys.FORMAT] != 'GLTF_SEPARATE':
        return gltf2_io_binary_data.BinaryData(data=image_data)
    return None

def gather_uri(image_data: bytes, image_name: str, mime_type: str, export_settings):
    if export_settings[gltf2_blender_export_keys.FORMAT] == 'GLTF_SEPARATE':
        return gltf2_io_image_data.ImageData(
            data=image_data,
            mime_type=mime_type,
            name=image_name,
        )
    return None


def gather_texture_hook(user_extension, gltf2_texture, blender_shader_sockets, export_settings):
    if not bpy.context.scene.VideoTextureProperties.enabled: return

    from io_scene_gltf2.blender.exp import gltf2_blender_search_node_tree
    from io_scene_gltf2.io.com import gltf2_io

    for blender_shader_socket in blender_shader_sockets:
        result = gltf2_blender_search_node_tree.from_socket(
            blender_shader_socket,
            gltf2_blender_search_node_tree.FilterByType(bpy.types.ShaderNodeTexImage)
        )
        if result:
            node = result[0].shader_node
            if node.image.source == "MOVIE":
                image = node.image
                data = tmp_encode_movie(image)
                mime_type = gather_mimetype(image)
                image_base_name, _extension = path.splitext(image.name)

                if mime_type is not None:
                    source = gltf2_io.Image(
                        buffer_view = gather_buffer_view(data, export_settings),
                        extensions = None,
                        extras = None,
                        mime_type = mime_type,
                        name = image_base_name,
                        uri = gather_uri(data, image_base_name, mime_type, export_settings)
                    )

                    extension = dict(source=source)
                    gltf2_texture.extensions[glTF_extension_name] = user_extension.Extension(
                        name = glTF_extension_name,
                        extension = extension,
                        required = extension_is_required
                    )